/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *    Combinations.java
 *    Copyright (C) 2002 Ola Leifler
 *
 */

package weka.core;

import java.io.*;
import java.util.*;

public class Combinations {

  class Combination extends Object implements Comparable {

    private Collection combination = null;

    public Combination(Collection c) {
      combination = new TreeSet();
      combination.addAll(c);
    }

    public int compareTo(Object o) {
      Collection c = ((Combination)o).getCollection();
      int compare = 0;
      Iterator cIter = c.iterator();
      Iterator iter=combination.iterator();
      for (;iter.hasNext()&&cIter.hasNext()&&compare==0; ) {
	compare = ((Integer)iter.next()).compareTo(cIter.next());
      } // end of for ()
      if ( compare == 0 ) {
	if ( cIter.hasNext()) {
	  compare = -1;
	} else if ( iter.hasNext() ) {
	  compare = 1;
	} // end of if ()
      } // end of if ()
      return compare;
    }

    public boolean equals(Object o) {
      Collection c = ((Combination)o).getCollection();
      boolean subSet = c.containsAll(combination);
      boolean superSet = combination.containsAll(c);
      return subSet && superSet;
    }

    public Collection getCollection(){
      return combination;
    }

    public int[] getCombination(){
      int[] combo = new int[combination.size()];
      int i=0;
      for (Iterator iter=combination.iterator();iter.hasNext();i++) {
	combo[i]=((Integer)iter.next()).intValue();
      } // end of for ()
      return combo;

    }

    public int hashCode(){
      return this.toString().hashCode();
    }

    public String toString() {
      StringBuffer s = new StringBuffer();
      /**
	 Append something informative to s 
      */
      for (Iterator iter=combination.iterator();iter.hasNext(); ) {
	s.append(((Integer)iter.next()).intValue() + (iter.hasNext()?",":""));
      } // end of for ()
      return s.toString();
    }
    

  }

  private int totalSet;
  private long numChoices;
  private boolean hasNext;
  private Combination currentCombo = null;
  private TreeSet oldCombos = null;

  public Combinations(int n,int k) throws Exception {
    if ( n < k ) {
      throw new Exception("Cannot select " + k + " objects out of " + n);
    } // end of if ()
    totalSet = n;   
    numChoices = Utils.nChooseK(n,k).longValue();
    TreeSet t = new TreeSet(); 
    while ( t.size() < k ) {
      t.add(new Integer((int)(totalSet*Math.random())));
    } // end of while ()
    currentCombo = new Combination(t);
    oldCombos = new TreeSet();
    oldCombos.add(currentCombo);
    hasNext = true;
  }

  public boolean hasNext() {
    return hasNext;
  }

  public int[] next() {
    int [] res = currentCombo.getCombination();
    hasNext = oldCombos.size() < numChoices;

    /**
       Calculate the next combination
     */

    if ( hasNext ) {
      do {
	TreeSet t = new TreeSet();
	while ( t.size() < res.length ) {
	  t.add(new Integer((int)(totalSet*Math.random())));
	} // end of while ()
	currentCombo = new Combination(t);
      } while ( oldCombos.contains(currentCombo) );
      oldCombos.add(currentCombo);
    } // end of if ()
    return res; 
  }

  public String toString() {
    return currentCombo.toString();
  }

  public static void main (String[] args) {
    int n=4,k=2;
    try {
        Combinations c = new Combinations(n,k);   
	System.out.println("Combinations of " + k + " objects out of " + n);
	while ( c.hasNext() ) {
	  System.out.println(c.toString());
	  c.next();
	} // end of while ()
    } catch (Exception e) {
      e.printStackTrace();
    } // end of try-catch
  } // end of main ()
  
}

