 /*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * AttributeValuePairs
 * Copyleft (C) 2002 Ola Leifler
 *
 */


package weka.roughset;

import weka.core.*;
import java.io.*;
import java.util.*;

public class AttributeValuePairs extends Object {
  
  private ArrayList pairs;
  private HashMap h;

  private void createMap(RoughInstances roughInstances) {
    h = new HashMap(size());
    for (int i=0;i<pairs.size();i++) {
      h.put((AttributeValuePair)pairs.get(i),new Integer(1));
    } // end of for ()
    for (int i=0;i<roughInstances.size();i++) {
      RoughInstance r = (RoughInstance)roughInstances.get(i);
      for (int j=0;j<r.numAttributes();j++) {
	AttributeValuePair p = new AttributeValuePair(r.attribute(j),
						      r.value(j));
	if ( h.containsKey(p) ) {
	  Integer val = (Integer)h.get(p);
	  h.put(p, new Integer(val.intValue()+1));
	} // end of if ()	
      } // end of for ()
    } // end of for ()
  }

  public AttributeValuePairs(){
    pairs = new ArrayList();
    h = null;
  }

  public AttributeValuePairs(AttributeValuePairs p){
    pairs = new ArrayList(p.getPairs());
    h = null;
  }

  public AttributeValuePairs(Instance i,Attributes a){
    pairs = new ArrayList(a.size());
    h = null;
    Iterator iter = a.iterator();
    while ( iter.hasNext() ) {
      Attribute attr = (Attribute)iter.next();
      AttributeValuePair p = new AttributeValuePair(attr,i.value(attr));
      pairs.add(p);
    } // end of while ()    
  }
  

  /* Create attribute-value pairs corresponding to all the decision
     classes of i */

  public AttributeValuePairs(Instances i) {
    pairs = new ArrayList();
    Enumeration e = i.enumerateInstances();
    while ( e.hasMoreElements() ) {
      Instance inst = (Instance)e.nextElement();
      AttributeValuePair p = new AttributeValuePair(inst);
      if ( !pairs.contains(p) ) {
	pairs.add(p);
      } // end of if ()
    } // end of while ()
    h = null;
  }

  public AttributeValuePair get(int index) {
    return (AttributeValuePair)pairs.get(index);
  }

  public ArrayList getPairs() {
    return pairs;
  }


  public AttributeValuePair getBestPair(RoughInstances r){
    if ( h == null ) {
      createMap(r);
    } // end of if ()
    AttributeValuePair best = null;
    int max = 0;
    Set keys = h.keySet();
    Iterator iter = keys.iterator();
    while (iter.hasNext()) {
      AttributeValuePair p = (AttributeValuePair)iter.next();
      if ( !p.getAttribute().equals(r.classAttribute()) ) {      
	int val = ((Integer)h.get(p)).intValue();
	if ( val > max ) {
	  max = val;
	  best = p;
	} // end of if ()
      } // end of if ()
    } // end of while ()
    return best;
  }


  public void addPair(Attribute a, double v) {
    AttributeValuePair p = new AttributeValuePair(a,v);
    if ( !pairs.contains(p)) {
      pairs.add(p);
    } // end of if ()
  }

  public void addPair(AttributeValuePair p) {
    if ( !pairs.contains(p)) {
      pairs.add(p);
    } // end of if ()
  }

  public void addPairs(RoughInstance i) {
    Iterator iter = i.getActiveAttributes().iterator();
    while ( iter.hasNext() ) {
      Attribute a = (Attribute)iter.next();
      AttributeValuePair p = new AttributeValuePair(a,i.value(a));
      if ( !pairs.contains(p)) {
	pairs.add(p);
      } // end of if ()
    } // end of while ()      
  }

  public void addPairs(AttributeValuePairs p) {
    Iterator i = p.iterator();
    while ( i.hasNext() ) {
      this.addPair((AttributeValuePair)i.next());
    } // end of while ()
  }

  public void removePair(Attribute a, double v) {
    AttributeValuePair p = new AttributeValuePair(a,v);
    this.removePair(p);
  }

  public void removePair(int index) {
    pairs.remove(index);
  }

  public void removePair(AttributeValuePair p) {
    Iterator i = pairs.iterator();
    while ( i.hasNext() ) {
      if ( ((AttributeValuePair)i.next()).equals(p) ) {
	i.remove();
      } // end of if ()
    } // end of while ()
  }

  public void removePairs(AttributeValuePairs p) {
    Iterator i = p.iterator();
    while ( i.hasNext() ) {
      this.removePair((AttributeValuePair)i.next());
    } // end of while ()
  }

  public void trim(RoughInstances b) {
    Iterator iter = this.iterator();
    while ( iter.hasNext() ) {
      
    } // end of while ()
    
  }

  public Attributes getAttributes(){
    Attributes a = new Attributes();
    Iterator i = iterator();
    while ( i.hasNext() ) {
      a.add(((AttributeValuePair)i.next()).getAttribute());
    } // end of while ()
    return a;
  }

  public boolean matches(Instance i){
    AttributeValuePair pair;
    Iterator iter = iterator();
    boolean comp = true;
    while ( iter.hasNext() ) {
      pair = (AttributeValuePair)iter.next();
      if ( Double.isNaN(pair.getValue()) || 
	   Double.isNaN(i.value(pair.getAttribute())) ) {
	/* NaN is used to indicate missing value or wildcard as we
	   interpret it, just as in WEKA's Instances */
	continue;
      } // end of if ()
      if ( i.value(pair.getAttribute()) != pair.getValue() ) {
	comp = false;
	break;
      } // end of if ()
    } // end of while ()
    return comp;
  }

  public double getPartialMatch(Instance i) {
    AttributeValuePair pair;
    Iterator iter = iterator();
    int matches = 0;
    while ( iter.hasNext() ) {
      pair = (AttributeValuePair)iter.next();
      if ( Double.isNaN(pair.getValue()) ) {
	/* NaN is used to indicate missing value or wildcard as we
	   interpret it, just as in WEKA's Instances */
	matches++;
	continue;
      } // end of if ()
      if ( i.value(pair.getAttribute()) == pair.getValue() ) {
	matches++;
      } // end of if ()
    } // end of while ()
    return (double)(matches/pairs.size());
  }

  public boolean equals(Object o) {
    boolean comp = true;
    if ( o instanceof AttributeValuePairs ) {
      AttributeValuePairs p = (AttributeValuePairs)o;
      return pairs.containsAll(p.getPairs()) && 
	p.getPairs().containsAll(pairs);
    } // end of if ()
    else {
      comp = false;
    } // end of else
    return comp;
  }

  public Iterator iterator() {
    return pairs.iterator();
  }

  public int size(){
    return pairs.size();
  }

  public boolean isEmpty() {
    return pairs.isEmpty();
  }

  public int hashCode() {
    return this.toString().hashCode();
  }

  public String toString(){
    StringBuffer s = new StringBuffer();
    Iterator i = iterator();
    while ( i.hasNext() ) {
       s.append(((AttributeValuePair)i.next()).toString());
       s.append("\n");
    } // end of while ()
    return s.toString();
  }

  public String ruleString(){
    StringBuffer s = new StringBuffer();
    Iterator i = iterator();
    while ( i.hasNext() ) {
       s.append(((AttributeValuePair)i.next()).ruleString());
       if ( i.hasNext() ) {
       s.append(", ");	 
       } // end of if ()
    } // end of while ()
    return s.toString();
  }


}


